@@ -22,6 +22,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "browse-dnd.h"
+
 #ifdef HAVE_CONFIG_H
 # include  <config.h>
 #endif /* HAVE_CONFIG_H */
@@ -33,15 +35,14 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include "browse-dnd.h"
 #include "debug.h"
 #include "errorchain.h"
 #include "gq-browser-node-dn.h"
 #include "gq-server-list.h"
 #include "gq-tab-browse.h"
+#include "gq-utilities.h"
 #include "ldapops.h"
 #include "ldif.h"
-#include "util.h"
 
 typedef struct _dnd_refresh {
      GQTreeWidget *tree;
@@ -72,20 +73,21 @@ static dnd_refresh *new_dnd_refresh_dn(GQTreeWidget *tree,
      return dr;
 }
 
-static dnd_refresh *new_dnd_refresh_node(GQTreeWidget *tree, 
-					 GQTreeWidgetNode *node,
-					 int options)
+static dnd_refresh*
+new_dnd_refresh_node(GQTreeWidget    * tree, 
+		     GQTreeWidgetNode* node,
+#warning "FIXME: options should be more specified"
+		     int               options)
 {
-     GqBrowserNodeDn *entry;
-     entry = GQ_BROWSER_NODE_DN(gq_tree_get_node_data (tree, node));
+	GqBrowserNodeDn* entry = (GqBrowserNodeDn*)gq_tree_get_node_data(tree, node);
+	gchar const    * dn = NULL;;
 
-     if (GQ_IS_BROWSER_NODE_DN(entry)) {
-	  return new_dnd_refresh_dn(tree, server_from_node(tree, node),
-				    entry->dn, options);
-     } else {
-	  return new_dnd_refresh_dn(tree, server_from_node(tree, node),
-				    NULL, options);
-     }
+	if(GQ_IS_BROWSER_NODE_DN(entry)) {
+		dn = gq_browser_node_dn_get_dn(entry);
+	}
+
+	return new_dnd_refresh_dn(tree, gq_browser_node_get_server(GQ_BROWSER_NODE(entry)),
+				  dn, options);
 }
 
 static void dnd_refresh_free(dnd_refresh *dr)
@@ -238,6 +240,7 @@ static void drag_begin(GtkWidget *ctreeroot,
 		       GdkDragContext *drag_context,
 		       GqTab *tab)
 {
+#ifndef USE_TREE_VIEW
      /* store data we began dragging with for drag_data_get later on */
      GQTreeWidgetNode *ctree_node;
      GqServer *server;
@@ -246,15 +249,14 @@ static void drag_begin(GtkWidget *ctreeroot,
      char cbuf[20];
 
      ctree_node = GQ_TAB_BROWSE(tab)->tree_row_selected;
-     server = server_from_node(GTK_CTREE(ctreeroot), ctree_node);
      entry = GQ_BROWSER_NODE(gq_tree_get_node_data (GTK_CTREE(ctreeroot),
 							  ctree_node));
+     server = gq_browser_node_get_server(entry);
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("drag_begin start node=%08lx entry=%08lx\n",
-		 (unsigned long) ctree_node,
-		 (unsigned long) entry);
+	  printf("drag_begin start node=%p entry=%p\n",
+		 ctree_node, entry);
      }
 #endif
 
@@ -262,12 +264,12 @@ static void drag_begin(GtkWidget *ctreeroot,
 	  GqBrowserNodeDn *dn_entry = GQ_BROWSER_NODE_DN(entry);
 
 	  seldata = g_hash_table_new(g_str_hash, g_str_equal);
-	  g_hash_table_insert(seldata, g_strdup("dn"), g_strdup(dn_entry->dn));
+	  g_hash_table_insert(seldata, g_strdup("dn"), g_strdup(gq_browser_node_dn_get_dn(dn_entry)));
 	  g_hash_table_insert(seldata, g_strdup("nickname"),
-			      g_strdup(server->name));
+			      g_strdup(gq_server_get_name(server)));
 	  g_hash_table_insert(seldata, g_strdup("server"),
-			      g_strdup(server->ldaphost));
-	  g_snprintf(cbuf, sizeof(cbuf), "%d", server->ldapport);
+			      g_strdup(gq_server_get_host(server)));
+	  g_snprintf(cbuf, sizeof(cbuf), "%d", gq_server_get_port(server));
 	  g_hash_table_insert(seldata, g_strdup("port"), g_strdup(cbuf));
 
 	  /* dragging is always done recusively (for now) */
@@ -280,11 +282,12 @@ static void drag_begin(GtkWidget *ctreeroot,
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("drag_begin %08lx %08lx\n", (unsigned long) ctreeroot,
-		 (unsigned long) seldata);
+	  printf("drag_begin %p %p\n", ctreeroot,
+		 seldata);
      }
 #endif
      }
+#endif
 }
 
 
@@ -355,7 +358,7 @@ static void drag_end(GtkWidget *ctreeroot,
      int ctx;
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("drag_end %08lx\n", (unsigned long) drag_context);
+	  printf("drag_end %p\n", drag_context);
      }
 #endif
      /* kill any motion timer ... */
@@ -409,8 +412,8 @@ static void drag_data_get(GtkWidget *ctreeroot,
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("drag_data_get ctx=%08lx data=%08lx\n", (unsigned long) drag_context,
-		 (unsigned long) data);
+	  printf("drag_data_get ctx=%p data=%p\n",
+		 drag_context, data);
      }
 #endif
 
@@ -424,8 +427,8 @@ static void drag_data_get(GtkWidget *ctreeroot,
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("data->data=%08lx data=%08lxx\n",
-		 (unsigned long)(data->data), (unsigned long) data);
+	  printf("data->data=%p data=%p\n",
+		 (data->data), data);
      }
 #endif
 }
@@ -483,7 +486,6 @@ static void do_move_after_reception(GtkWidget *ctreeroot,
 				    GqServer *target_server,
 				    int flags)
 {
-     GQTreeWidgetNode *ctree_node;
      char *dn = NULL;
      GqServer *source_server;
      char *newdn = NULL;
@@ -492,8 +494,8 @@ static void do_move_after_reception(GtkWidget *ctreeroot,
      context = error_new_context(_("Moving entry"), ctreeroot);
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("do_move_after_reception selhash=%08lx server=%s dn=%s\n",
-		 (unsigned long) selhash,
+	  printf("do_move_after_reception selhash=%p server=%s dn=%s\n",
+		 selhash,
 		 (const char*)g_hash_table_lookup(selhash, "nickname"), dn);
      }
 #endif
@@ -518,7 +520,7 @@ static void do_move_after_reception(GtkWidget *ctreeroot,
      
 
      if ((source_server == target_server) &&
-	 ( strcasecmp(dn, target_entry->dn) == 0)) {
+	 ( strcasecmp(dn, gq_browser_node_dn_get_dn(target_entry)) == 0)) {
           error_push(context, _("Cannot move/copy entry onto itself!"));
 	  
 	  goto done;
@@ -526,7 +528,7 @@ static void do_move_after_reception(GtkWidget *ctreeroot,
      
      if ((flags & MOVE_RECURSIVELY) &&
 	 (source_server == target_server) &&
-	 is_ancestor(target_entry->dn, dn)) {
+	 is_ancestor(gq_browser_node_dn_get_dn(target_entry), dn)) {
           error_push(context,
 		     _("Cannot recursively move/copy entry onto or below itself!"));
 	  
@@ -535,7 +537,7 @@ static void do_move_after_reception(GtkWidget *ctreeroot,
      
      if (GQ_IS_BROWSER_NODE_DN(target_entry)) {
 	  newdn = move_entry(dn, source_server, 
-			     target_entry->dn, target_server,
+			     gq_browser_node_dn_get_dn(target_entry), target_server,
 			     flags, move_progress, context);
 
 	  /* register that we have to refresh the target node */
@@ -543,7 +545,7 @@ static void do_move_after_reception(GtkWidget *ctreeroot,
 	       g_list_append(dnd_refresh_list,
 			     new_dnd_refresh_dn(GTK_CTREE(ctreeroot),
 						target_server, 
-						target_entry->dn,
+						gq_browser_node_dn_get_dn(target_entry),
 						REFRESH_FORCE_EXPAND)
 			     );
 
@@ -561,6 +563,7 @@ static void do_move_after_reception(GtkWidget *ctreeroot,
 /*  				       REFRESH_FORCE_EXPAND); */
 	  
 	  if (newdn) {
+#ifndef USE_TREE_VIEW
 	       ctree_node = node_from_dn(GTK_CTREE(ctreeroot),
 					 target_ctree_node, newdn);
 	       if (ctree_node)
@@ -572,9 +575,12 @@ static void do_move_after_reception(GtkWidget *ctreeroot,
 	       if (debug & GQ_DEBUG_BROWSER_DND) {
 		    printf("moved %s below %s, became %s\n",
 			   dn,
-			   target_entry->dn,
+			   gq_browser_node_dn_get_dn(target_entry),
 			   newdn);
 	       }
+#endif
+#else
+	       g_message("FIXME: implement with GtkTreeView");
 #endif
 	       g_free(newdn);
 	  }
@@ -606,8 +612,8 @@ static void drag_data_received(GtkWidget *ctreeroot,
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("drag_data_received ctx=%08lx seldata=%08lx\n",
-		 (unsigned long) drag_context, (unsigned long) data);
+	  printf("drag_data_received ctx=%p seldata=%p\n",
+		 drag_context, data);
      }
 #endif
 
@@ -628,7 +634,9 @@ static void drag_data_received(GtkWidget *ctreeroot,
 	servers... */
 
      answer =
-	  question_popup(_("Do you really want to move this entry?"),
+#warning "FIXME: set a parent window"
+	  question_popup(NULL,
+			 _("Do you really want to move this entry?"),
 			 _("Do you really want to move this entry recursively?\n"
 			   "Note that you should only do this if you have\n"
 			   "access to ALL attributes of the object to move\n"
@@ -686,8 +694,8 @@ static void drag_data_delete(GtkWidget *widget,
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("drag_data_delete ctx=%08lx suggested=%d action=%d server=%s dn=%s\n",
-		 (unsigned long) drag_context,
+	  printf("drag_data_delete ctx=%p suggested=%d action=%d server=%s dn=%s\n",
+		 drag_context,
 		 drag_context->suggested_action,
 		 drag_context->action,
 		 s, dn
@@ -725,30 +733,30 @@ static void drag_data_delete(GtkWidget *widget,
 			    "drag-and-selection-data");
 }
 
-void copy_entry(GtkWidget *widget, GqTab *tab)
+void
+copy_entry(GtkWidget *widget, GqTab *tab)
 {
      GQTreeWidget *ctree;
      int have_sel;
      GqServer *server;
      GqBrowserNodeDn *entry;
-     GQTreeWidgetNode *ctree_node;
      GHashTable *seldata;
      char cbuf[20];
 
      ctree = GQ_TAB_BROWSE(tab)->ctreeroot;
 
-     ctree_node = GQ_TAB_BROWSE(tab)->tree_row_popped_up;
-     entry = GQ_BROWSER_NODE_DN(gq_tree_get_node_data (ctree, ctree_node));
+	entry = gq_browser_model_get_node(gtk_tree_view_get_model(GTK_TREE_VIEW(ctree)),
+					  &GQ_TAB_BROWSE(tab)->tree_row_popped_up);
      if (entry == NULL) return;
      if (!GQ_IS_BROWSER_NODE_DN(entry)) return;
 
-     server = server_from_node(ctree, ctree_node);
+	server = gq_browser_node_get_server(entry);
 
      seldata = g_hash_table_new(g_str_hash, g_str_equal);
-     g_hash_table_insert(seldata, g_strdup("dn"), g_strdup(entry->dn));
-     g_hash_table_insert(seldata, g_strdup("nickname"), g_strdup(server->name));
-     g_hash_table_insert(seldata, g_strdup("server"), g_strdup(server->ldaphost));
-     g_snprintf(cbuf, sizeof(cbuf), "%d", server->ldapport);
+     g_hash_table_insert(seldata, g_strdup("dn"), g_strdup(gq_browser_node_dn_get_dn(entry)));
+     g_hash_table_insert(seldata, g_strdup("nickname"), g_strdup(gq_server_get_name(server)));
+     g_hash_table_insert(seldata, g_strdup("server"), g_strdup(gq_server_get_host(server)));
+     g_snprintf(cbuf, sizeof(cbuf), "%d", gq_server_get_port(server));
      g_hash_table_insert(seldata, g_strdup("port"), g_strdup(cbuf));
 
      gtk_object_set_data_full(GTK_OBJECT(ctree), 
@@ -762,7 +770,7 @@ void copy_entry(GtkWidget *widget, GqTab *tab)
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("copy_entry %08lx %d\n", (unsigned long) ctree, have_sel);
+	  printf("copy_entry %p %d\n", ctree, have_sel);
      }
 #endif
 
@@ -785,7 +793,7 @@ void copy_entry_all(GtkWidget *widget, GqTab *tab)
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("copy_entry_all %08lx\n", (unsigned long) widget);
+	  printf("copy_entry_all %p\n", widget);
      }
 #endif
 
@@ -920,7 +928,7 @@ static void get_selection_gq(GtkWidget *ctree,
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("get_selection_gq data=%08lx\n", (unsigned long) data);
+	  printf("get_selection_gq data=%p\n", data);
      }
 #endif
 
@@ -932,8 +940,8 @@ static void get_selection_gq(GtkWidget *ctree,
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("data->data=%08lx data=%08lxx\n",
-		 (unsigned long)(data->data), (unsigned long) data);
+	  printf("data->data=%p data=%p\n",
+		 (data->data), data);
      }
 #endif
 
@@ -970,7 +978,7 @@ void paste_entry(GtkWidget *widget, GqTab *tab)
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("paste_entry %08lx\n", (unsigned long) ctree);
+	  printf("paste_entry %p\n", ctree);
      }
 #endif
      gtk_selection_convert(ctree,
@@ -986,12 +994,10 @@ void paste_entry(GtkWidget *widget, GqTab *tab)
 
 /* gets fired after the gtk_selection_convert triggered through
    "paste" finished. Similar to when dragging stops. */
-static void selection_received(GtkWidget *ctree,
-			       GtkSelectionData *data,
-			       guint time,
-			       GqTab *tab) 
+static void
+selection_received(GqTab           * tab,
+		   GtkSelectionData* data) 
 {
-     GQTreeWidgetNode *ctree_node;
      GqBrowserNodeDn *target_entry;
      GqServer *target_server;
      GQTreeWidget *ctreeroot;
@@ -1001,19 +1007,18 @@ static void selection_received(GtkWidget *ctree,
 
 #ifdef DEBUG
      if (debug & GQ_DEBUG_BROWSER_DND) {
-	  printf("selection_received seldata=%08lx\n",
-		 (unsigned long) data);
+	  printf("selection_received seldata=%p\n",
+		 data);
      }
 #endif
 
-     ctree_node = GQ_TAB_BROWSE(tab)->tree_row_popped_up;
-     target_entry =
-	  GQ_BROWSER_NODE_DN(gq_tree_get_node_data (ctreeroot,
-						     ctree_node));
+	target_entry = gq_browser_model_get_node(gtk_tree_view_get_model(GTK_TREE_VIEW(ctreeroot)),
+						 &GQ_TAB_BROWSE(tab)->tree_row_popped_up);
+
      if (target_entry == NULL) return;
      if (!GQ_IS_BROWSER_NODE_DN(target_entry)) return;
 
-     target_server = server_from_node(ctreeroot, ctree_node);
+	target_server = gq_browser_node_get_server(target_entry);
 
      /* For now we allow recursive moves, as we _believe_ it
 	works even with a limited number of answers from
@@ -1045,13 +1050,15 @@ static void selection_received(GtkWidget *ctree,
 		       "access to ALL attributes of the object.\n");
 	  }
 	  
-	  answer = 
-	       question_popup(_("Do you really want to move this entry?"),
+	  answer =
+#warning "FIXME: set a good parent window"
+	       question_popup(NULL,
+			      _("Do you really want to move this entry?"),
 			      msg);
 	  
 	  if (answer) {
 	       do_move_after_reception(GTK_WIDGET(ctreeroot),
-				       selhash, ctree_node,
+				       selhash, &GQ_TAB_BROWSE(tab)->tree_row_popped_up,
 				       target_entry, target_server,
 				       flags);
 	  }
@@ -1077,10 +1084,8 @@ void browse_dnd_setup(GtkWidget *ctreeroot, GqTab *tab)
 			"selection-get",
 			G_CALLBACK(get_selection),
 			tab);
-     g_signal_connect(ctreeroot, 
-			"selection-received",
-			G_CALLBACK(selection_received),
-			tab);
+	g_signal_connect_swapped(ctreeroot, "selection-received",
+				 G_CALLBACK(selection_received), tab);
 
      target_entry = (GtkTargetEntry*) g_malloc(sizeof(GtkTargetEntry));
      target_entry->target = "gq-browse-ctree";
