@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: configfile.c 975 2006-09-07 18:44:41Z herzi $ */
+/* $Id$ */
 
 #include <stdio.h>
 #include <string.h>
@@ -39,34 +39,25 @@
 #include <glib/gi18n.h>
 
 #include "common.h"
-#include "configfile.h"
-#include "gq-keyring.h"
-#include "gq-server-list.h"
-#include "template.h"
-#include "util.h"
-#include "filter.h"
+#include "debug.h"
 #include "encode.h"		/* gq_codeset */
 #include "errorchain.h"
-#include "debug.h"
+#include "filter.h"
+#include "gq-keyring.h"
+#include "gq-server-list.h"
+#include "gq-utilities.h"
+#include "gq-xml.h"
 #include "ldif.h"  /* for b64_encode */
 #include "syntax.h"
-#include "gq-xml.h"
+#include "template.h"
 
 struct gq_config *config;
 GList *transient_servers = NULL;
 
-const struct tokenlist token_searchargument[] = {
-     { SEARCHARG_BEGINS_WITH, "Begins with",	NULL },
-     { SEARCHARG_ENDS_WITH,   "Ends with",	NULL },
-     { SEARCHARG_CONTAINS,    "Contains",	NULL },
-     { SEARCHARG_EQUALS,      "Equals",		NULL },
-     { 0, "",					NULL }
-};
-
 const struct tokenlist token_bindtype[] = {
-     { BINDTYPE_SIMPLE,    "Simple",		NULL },
-     { BINDTYPE_KERBEROS,  "Kerberos",		NULL },
-     { BINDTYPE_SASL,      "SASL",		NULL },
+     { GQ_BIND_SIMPLE,    "Simple",		NULL },
+     { GQ_BIND_KERBEROS,  "Kerberos",		NULL },
+     { GQ_BIND_SASL,      "SASL",		NULL },
      { 0, "",					NULL }
 };
 
@@ -312,16 +303,18 @@ void config_write_int(struct writeconfig *wc, int value, const char *entity,
 }
 
 
-void config_write_string(struct writeconfig *wc,
-			 const char *value,
-			 const char *entity, GHashTable *attr)
+void
+config_write_string(struct writeconfig* wc,
+		    const char        * value,
+		    const char        * entity,
+		    GHashTable        * attr)
 {
      GString *outstr = g_string_sized_new(1024);
      gunichar c;
 
      start_tag(outstr, entity, attr);
 
-     for(c = g_utf8_get_char(value); c ;
+     for(c = value ? g_utf8_get_char(value) : '\0'; c ;
 	 value = g_utf8_next_char(value), c = g_utf8_get_char(value)) {
 	  switch(c) {
 	  case '<':
@@ -384,51 +377,40 @@ static void attrs_dumper(char *key,
 }
 
 static void
-write_server(GQServerList* list, GqServer* server, gpointer user_data) {
-	gpointer* wc_and_cfg = user_data;
-	struct writeconfig* wc = wc_and_cfg[0];
-	struct gq_config * cfg = wc_and_cfg[1];
+write_server(GQServerList* list      G_GNUC_UNUSED,
+	     GqServer    * server,
+	     gpointer      user_data)
+{
+	struct writeconfig* wc = user_data;
 
 /*	  GString *pw = g_string_new(); */
 	  config_write(wc, "<ldapserver>\n");
 	  wc->indent++;
 
-	  config_write_string(wc, server->name, "name", NULL);
-	  config_write_string(wc, server->ldaphost, "ldaphost", NULL);
-	  config_write_int(wc, server->ldapport, "ldapport", NULL);
-	  config_write_string_ne(wc, server->basedn, "basedn", NULL);
-	  config_write_string_ne(wc, server->binddn, "binddn", NULL);
+	config_write_string(wc, gq_server_get_name(server), "name", NULL);
+	config_write_string(wc, gq_server_get_host(server), "ldaphost", NULL);
+	config_write_int   (wc, gq_server_get_port(server), "ldapport", NULL);
+	config_write_string_ne(wc, server->basedn, "basedn", NULL);
+	config_write_string_ne(wc, gq_server_get_bind_dn(server), "binddn", NULL);
 
-	  if (cfg->config_version == 0) {
-	       config_write_string_ne(wc, server->bindpw, "bindpw", NULL);
-	  } else if(server->bindpw && *server->bindpw) {
-	       gq_keyring_save_server_password(server);
-	  }
-
-	  if(server->bindtype != DEFAULT_BINDTYPE)
-	       config_write_string_ne(wc, detokenize(token_bindtype, server->bindtype), "bindtype", NULL);
-	  config_write_string_ne(wc, server->searchattr,
-				 "search-attribute", NULL);
-	  if(server->maxentries != DEFAULT_MAXENTRIES)
-	       config_write_int(wc, server->maxentries, "maxentries", NULL);
-	  if(server->cacheconn != DEFAULT_CACHECONN)
-	       config_write_bool(wc, server->cacheconn, 
-				 "cache-connection", NULL);
-	  if(server->enabletls != DEFAULT_ENABLETLS)
-	       config_write_bool(wc, server->enabletls, "enable-tls", NULL);
+	config_write_string_ne(wc, detokenize(token_bindtype, gq_server_get_bind_type(server)),
+			       "bindtype", NULL);
+	config_write_string_ne(wc, gq_server_get_search_attribute(server),
+			       "search-attribute", NULL);
+	config_write_bool(wc, gq_server_get_cache_connection(server),
+			  "cache-connection", NULL);
+	config_write_bool(wc, gq_server_get_use_tls(server), "enable-tls", NULL);
 	  if(server->local_cache_timeout != DEFAULT_LOCAL_CACHE_TIMEOUT)
 	       config_write_int(wc, server->local_cache_timeout, 
 				"local-cache-timeout", NULL);
-	  if(server->ask_pw != DEFAULT_ASK_PW)
-	       config_write_bool(wc, server->ask_pw, "ask-pw", NULL);
-	  if(server->hide_internal != DEFAULT_HIDE_INTERNAL)
-	       config_write_bool(wc, server->hide_internal, 
-				 "hide-internal", NULL);
-	  if(server->show_ref != DEFAULT_SHOW_REF)
+	config_write_bool(wc, gq_server_get_ask_pw(server), "ask-pw", NULL);
+	config_write_bool(wc, gq_server_get_hide_internals(server), 
+			  "hide-internal", NULL);
+	if(server->show_ref != DEFAULT_SHOW_REF)
 	       config_write_bool(wc, server->show_ref, "show-ref", NULL);
 
-	  wc->indent--;
-	  config_write(wc, "</ldapserver>\n\n");
+	wc->indent--;
+	config_write(wc, "</ldapserver>\n\n");
 }
 
 static gboolean save_config_internal(int write_context,
@@ -436,19 +418,13 @@ static gboolean save_config_internal(int write_context,
 				     const char *rcpath)
 {
      GList *templatelist, *oclist, *filterlist;
-     GqServer *server;
      struct gq_template *template;
      struct gq_filter *filter;
      struct writeconfig *wc;
      char *tmprcpath;
      struct stat sfile;
      int mode = S_IRUSR|S_IWUSR;
-     GList *I;
      gboolean ok = FALSE;
-     gpointer wc_and_cfg[2] = {
-	     NULL,
-	     cfg
-     };
 
      if (cfg->config_version > CURRENT_CONFIG_VERSION) {
 	  error_push(write_context,
@@ -457,7 +433,6 @@ static gboolean save_config_internal(int write_context,
      }
 
      wc = new_writeconfig();
-     wc_and_cfg[0] = wc;
 
      /* write to temp file... */
      tmprcpath = g_malloc(strlen(rcpath) + 10);
@@ -494,9 +469,6 @@ static gboolean save_config_internal(int write_context,
      }
 
      config_write_bool(wc, cfg->confirm_mod, "confirm-mod", NULL);
-     config_write_string(wc, detokenize(token_searchargument, 
-					cfg->search_argument),
-			 "search-argument", NULL);
      config_write_bool(wc, cfg->showdn, "show-dn", NULL);
      config_write_bool(wc, cfg->showoc, "show-oc", NULL);
      config_write_bool(wc, cfg->show_rdn_only, "show-rdn-only", NULL);
@@ -524,7 +496,7 @@ static gboolean save_config_internal(int write_context,
      config_write(wc, "\n");
 
      /* ldapservers */
-     gq_server_list_foreach(gq_server_list_get(), write_server, wc_and_cfg);
+     gq_server_list_foreach(gq_server_list_get(), write_server, wc);
 
      /* templates */
      templatelist = cfg->templates;
@@ -651,7 +623,6 @@ struct gq_config *new_config(void)
      cfg->filters   = NULL;
 
      cfg->confirm_mod = 0;
-     cfg->search_argument = SEARCHARG_BEGINS_WITH;
      cfg->showdn = TRUE;
      cfg->showoc = TRUE;
      cfg->show_rdn_only = TRUE;
@@ -694,8 +665,11 @@ void init_config(void)
      if (config->config_version < CURRENT_CONFIG_VERSION
 	 && (config->asked_version < CURRENT_CONFIG_VERSION ||
 	     (config->asked_version == CURRENT_CONFIG_VERSION &&
-	      (time(NULL) - config->last_asked) > 31*86400))) {
-	  int rc = question_popup(_("Upgrade configuration?"),
+	      (time(NULL) - config->last_asked) > 31*86400)))
+	{
+#warning "FIXME: set a parent window"
+	  int rc = question_popup(NULL,
+				  _("Upgrade configuration?"),
 				  _("Do you want to upgrade to the latest configuration file version?\nIf you say no you may not be able to use all functionalities.\nIf you say yes you may not be able to use your configuration with older versions of gq.\n"));
 
 	  config->asked_version = CURRENT_CONFIG_VERSION;
@@ -709,16 +683,9 @@ void init_config(void)
 
      if(!gq_server_list_n_servers(list)) {
 	  /* no ldapserver defined in configfile */
-	  default_server = gq_server_new();
-
-	  g_free(default_server->name);
-	  g_free(default_server->ldaphost);
-	  g_free(default_server->searchattr);
-	  default_server->name = g_strdup("localhost");
-	  default_server->ldaphost = g_strdup("localhost");
-	  default_server->searchattr = g_strdup("cn");
-	  default_server->ldapport = 389;
+	  default_server = gq_server_new(NULL);
 
+	  gq_server_set_name(default_server, "localhost");
 	  gq_server_list_add(list, default_server);
      }
 
@@ -728,7 +695,7 @@ void init_config(void)
 
 static gboolean attr_remove(gpointer key,
 			    gpointer value,
-			    gpointer user_data)
+			    gpointer user_data G_GNUC_UNUSED)
 {
      g_free(key);
      free_attr_settings((struct attr_settings *)value);
@@ -761,17 +728,11 @@ const char *human_readable_attrname(const char *attrname)
 }
 
 void free_config(struct gq_config *cfg) {
-     GqServer *server;
-     GList *I;
-
-     g_assert(cfg);
+     g_return_if_fail(cfg);
 
      if (cfg->schemaserver) g_free(cfg->schemaserver);
      cfg->schemaserver = NULL;
 
-     /* free malloc'd ldapservers */
-     g_object_unref(gq_server_list_get());
-
      /* free templates */
      g_list_foreach(cfg->templates, (GFunc) free_template, NULL);
      g_list_free(cfg->templates);
