@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: debug.h 881 2006-05-04 08:29:02Z herzi $ */
+/* $Id$ */
 
 #ifndef GQ_DEBUG_H_INCLUDED
 #define GQ_DEBUG_H_INCLUDED
@@ -64,7 +64,7 @@ extern int debug;
 #    include <malloc.h>
 #    include <gtk/gtk.h>
 
-void init_memstat_timer();
+void init_memstat_timer(void);
 
 #  endif /* HAVE_MALLINFO */
 #endif /* DEBUG */
