@@ -22,12 +22,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: dt_entry.h 937 2006-05-25 13:51:47Z herzi $ */
+/* $Id$ */
 
 #ifndef DT_ENTRY_H_INCLUDED
 #define DT_ENTRY_H_INCLUDED
 
-#include "formfill.h"
+#include "gq-type-display.h"
 #include "syntax.h"
 
 typedef GQTypeDisplay            GQDisplayEntry;
@@ -48,11 +48,11 @@ struct _dt_entry_handler {
 
 /* Methods, only to be used by subclasses */
 GtkWidget *dt_entry_get_widget(int error_context,
-			       struct formfill *form, GByteArray *data,
-			       GCallback *activatefunc,
+			       GqFormfill *form, GByteArray *data,
+			       GCallback  activatefunc,
 			       gpointer funcdata);
-GByteArray *dt_entry_get_data(struct formfill *form, GtkWidget *widget);
-void dt_entry_set_data(struct formfill *form, GByteArray *data,
+GByteArray *dt_entry_get_data(GqFormfill *form, GtkWidget *widget);
+void dt_entry_set_data(GqFormfill *form, GByteArray *data,
 		       GtkWidget *widget);
 
 #endif
