@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: dt_int.c 952 2006-08-26 11:17:35Z herzi $ */
+/* $Id$ */
 
 #include <string.h>
 #include <sys/types.h>
@@ -42,15 +42,14 @@
 #include "dt_int.h"
 
 #include "common.h"
-#include "util.h"
-#include "errorchain.h"
-#include "gq-tab-browse.h"
-#include "input.h"
-#include "tinput.h"
+#include "dtutil.h"
 #include "encode.h"
+#include "errorchain.h"
+#include "gq-input-form.h"
+#include "gq-tab-browse.h"
+#include "gq-utilities.h"
 #include "ldif.h" /* for b64_decode */
 #include "syntax.h"
-#include "dtutil.h"
 
 static void dt_int_verify(GtkWidget *entry)
 {
@@ -103,8 +102,8 @@ static void dt_int_verify(GtkWidget *entry)
 }
 
 GtkWidget *dt_int_get_widget(int error_context,
-			     struct formfill *form, GByteArray *data,
-			     GCallback *activatefunc,
+			     GqFormfill *form, GByteArray *data,
+			     GCallback  activatefunc,
 			     gpointer funcdata)
 {
     GtkWidget *hbox, *inputbox, *label;
@@ -136,15 +135,15 @@ GtkWidget *dt_int_get_widget(int error_context,
     return hbox;
 }
 
-GByteArray *dt_int_get_data(struct formfill *form, GtkWidget *widget)
+GByteArray *dt_int_get_data(GqFormfill *form, GtkWidget *widget)
 {
     return editable_get_text(GTK_EDITABLE(dt_int_retrieve_inputbox(widget)));
 }
 
-void dt_int_set_data(struct formfill *form, GByteArray *data,
+void dt_int_set_data(GqFormfill *form, GByteArray *data,
 		       GtkWidget *widget)
 {
-     GQTypeDisplayClass* klass = g_type_class_ref(form->dt_handler);
+     GQTypeDisplayClass* klass = g_type_class_ref(gq_formfill_get_dt_handler(form));
      editable_set_text(GTK_EDITABLE(dt_int_retrieve_inputbox(widget)),
 		       data,
 		       DT_INT(klass)->encode,
