@@ -24,7 +24,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: dt_jpeg.h 937 2006-05-25 13:51:47Z herzi $ */
+/* $Id$ */
 
 #ifndef DT_JPEG_H_INCLUDED
 #define DT_JPEG_H_INCLUDED
@@ -40,17 +40,17 @@ typedef GQDisplayBinaryClass GQDisplayJPEGClass;
 GType gq_display_jpeg_get_type(void);
 
 /* Methods, only to be used by subclasses */
-GtkWidget *dt_jpeg_get_data_widget(struct formfill *form,
-				   GCallback *activatefunc,
+GtkWidget *dt_jpeg_get_data_widget(GqFormfill *form,
+				   GCallback  activatefunc,
 				   gpointer funcdata);
-GByteArray *dt_jpeg_get_data(struct formfill *form, GtkWidget *widget);
+GByteArray *dt_jpeg_get_data(GqFormfill *form, GtkWidget *widget);
 
-void dt_jpeg_store_data(struct formfill *form,
+void dt_jpeg_store_data(GqFormfill *form,
 			GtkWidget *hbox,
 			GtkWidget *data_widget,
 			const GByteArray *data);
 
-void dt_jpeg_delete_data(struct formfill *form,
+void dt_jpeg_delete_data(GqFormfill *form,
 			 GtkWidget *hbox,
 			 GtkWidget *data_widget);
 
