@@ -22,13 +22,11 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: dt_oc.h 937 2006-05-25 13:51:47Z herzi $ */
+/* $Id$ */
 
 #ifndef DT_OC_H_INCLUDED
 #define DT_OC_H_INCLUDED
 
-#include "formfill.h"
-#include "syntax.h"
 #include "dt_entry.h"
 
 typedef GQDisplayEntry      GQDisplayOC;
@@ -41,11 +39,11 @@ GType gq_display_oc_get_type(void);
 
 /* Methods, only to be used by subclasses */
 GtkWidget *dt_oc_get_widget(int error_context,
-			    struct formfill *form, GByteArray *data,
-			    GCallback *activatefunc,
+			    GqFormfill *form, GByteArray *data,
+			    GCallback  activatefunc,
 			    gpointer funcdata);
-GByteArray *dt_oc_get_data(struct formfill *form, GtkWidget *widget);
-void dt_oc_set_data(struct formfill *form, GByteArray *data,
+GByteArray *dt_oc_get_data(GqFormfill *form, GtkWidget *widget);
+void dt_oc_set_data(GqFormfill *form, GByteArray *data,
 		    GtkWidget *widget);
 
 #endif
