@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: dt_text.h 937 2006-05-25 13:51:47Z herzi $ */
+/* $Id$ */
 
 #ifndef DT_TEXT_H_INCLUDED
 #define DT_TEXT_H_INCLUDED
@@ -39,8 +39,8 @@ GType gq_display_text_get_type(void);
 
 /* Methods, only to be used by subclasses */
 GtkWidget *dt_text_get_widget(int error_context,
-			      struct formfill *form, GByteArray *data,
-			      GCallback *activatefunc,
+			      GqFormfill *form, GByteArray *data,
+			      GCallback  activatefunc,
 			      gpointer funcdata);
 
 #define DEFAULT_LINES 5
