@@ -1,48 +1,56 @@
-/*
-    GQ -- a GTK-based LDAP client
-    Copyright (C) 1998-2003 Bert Vermeulen
-    Copyright (C) 2002-2003 Peter Stamfest
+/* This file is part of GQ
+ *
+ * AUTHORS
+ *     Sven Herzberg  <herzi@gnome-de.org>
+ *
+ * Copyright (C) 1998-2003 Bert Vermeulen
+ * Copyright (C) 2002-2003 Peter Stamfest
+ * Copyright (C) 2007  Sven Herzberg
+ *
+ * This program is released under the Gnu General Public License with
+ * the additional exemption that compiling, linking, and/or using
+ * OpenSSL is allowed.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * USA
+ */
 
-    This program is released under the Gnu General Public License with
-    the additional exemption that compiling, linking, and/or using
-    OpenSSL is allowed.
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
-
-#include <glib/gi18n.h>
-#include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
+#include "errorchain.h"
 
 #include <stdio.h>
 #include <string.h>
 
-#include "errorchain.h"
+#include <glib/gi18n.h>
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
+
 #include "common.h"
-#include "util.h"
 #include "debug.h"
-#include "input.h"
 #include "encode.h"
-#include "mainwin.h"		/* message_log_append */
+#include "gq-input-form.h"
+#include "gq-main-loop.h"
+#include "gq-utilities.h"
+#include "gq-window.h" // message_log_append
 
 static struct errchain *error_chain_by_context(int q);
 
 static GList *chains = NULL;
-static int error_context = 0;
+static int error_context = 1;
 
-static struct errchain *new_errchain()
+static struct errchain*
+new_errchain(void)
 {
      struct errchain *new_chain;
      new_chain = g_malloc0(sizeof(struct errchain));
@@ -51,7 +59,8 @@ static struct errchain *new_errchain()
      return new_chain;
 }
 
-static void free_errchain(struct errchain *chain)
+static void
+free_errchain(struct errchain *chain)
 {
      if (chain) {
 	  g_free(chain->title);
@@ -64,7 +73,9 @@ static void free_errchain(struct errchain *chain)
      }
 }
 
-int error_new_context(const char *title, GtkWidget *modal_for)
+int
+error_new_context(const char* title,
+		  GtkWidget * modal_for)
 {
      struct errchain *new_chain;
 
@@ -81,18 +92,33 @@ int error_new_context(const char *title, GtkWidget *modal_for)
      new_chain->transient_for = modal_for;
      if (modal_for) gtk_widget_ref(modal_for);
 
+     if(!new_chain->context) {
+	     g_warning("Created chain had invalid context 0");
+	     free_errchain(new_chain);
+	     return 0;
+     }
+
      return new_chain->context;
 }
 
 
 
-static void error_push_production_v(int context, const char *fmt, va_list ap)
+static void
+error_push_production_v(int         context,
+			const char* fmt,
+			va_list     ap)
 {
      struct errchain *chain;
      GString *str;
      int n, a, retry = 0;
+
+	if(!context) {
+		return;
+	}
+
      a = strlen(fmt) + 50;
-     str = g_string_sized_new(a);  /* used for glib1 compatibility */
+	str = g_string_sized_new(a);
+#warning "FIXME: this is so sick"
 
      /* I hope it is ok to repeatedly use ap like this */
      do {
@@ -122,19 +148,36 @@ static void error_push_production_v(int context, const char *fmt, va_list ap)
      g_string_free(str, FALSE);
 }
 
-void error_push_production(int context, const char *fmt, ...)
+void
+error_push_production(int         context,
+		      const char* fmt,
+		      ...)
 {
      va_list ap;
+
+	if(!context) {
+		return;
+	}
+
      va_start(ap, fmt);
      error_push_production_v(context, fmt, ap);
      va_end(ap);
 }
 
 #ifdef DEBUG
-void error_push_debug(const char *file, int line, 
-		      int context, const char *fmt, ...)
+void
+error_push_debug(const char *file,
+		 int line, 
+		 int context,
+		 const char *fmt,
+		 ...)
 {
      va_list ap;
+
+	if(!context) {
+		return;
+	}
+
      va_start(ap, fmt);
 
      if (debug & GQ_DEBUG_ERROR_LINE) {
@@ -161,10 +204,15 @@ void error_push_debug(const char *file, int line,
  * Check LDAP connection for additional error information and append
  * it to the error context if such information is available.
  */
-void push_ldap_addl_error(LDAP *ld, int context)
+void
+push_ldap_addl_error(LDAP *ld, int context)
 {
      char *error_msg;
 
+	if(!context) {
+		return;
+	}
+
      ldap_get_option(ld, LDAP_OPT_ERROR_STRING, &error_msg);
      if (error_msg != NULL && *error_msg != 0) {
 	  error_push(context, _("Additional error: %s"), error_msg);
@@ -172,11 +220,14 @@ void push_ldap_addl_error(LDAP *ld, int context)
 }
 
 /* returns chain for requested context */
-static struct errchain *error_chain_by_context(int q)
+static struct errchain*
+error_chain_by_context(int q)
 {
      GList *I;
      struct errchain *chain;
 
+	g_return_val_if_fail(q, NULL);
+
      for (I = chains ; I ; I = g_list_next(I)) {
 	  chain = I->data;
 	  if (chain->context == q) return chain;
@@ -186,34 +237,50 @@ static struct errchain *error_chain_by_context(int q)
      return NULL; /* make sure the compiler shuts up */
 }
 
-static void error_free(int context)
+static void
+error_free(int context)
 {
      struct errchain *chain;
 
+	if(!context) {
+		return;
+	}
+
      chain = error_chain_by_context(context);
-     g_assert(chain);
+	g_return_if_fail(chain);
 
      chains = g_list_remove(chains, chain);
      
      free_errchain(chain);
 }
 
-void error_clear(int context)
+void
+error_clear(int context)
 {
-     struct errchain *chain = error_chain_by_context(context);
-     g_assert(chain);
+     struct errchain *chain;
+
+	if(!context) {
+		return;
+	}
+	chain = error_chain_by_context(context);
+	g_assert(chain);
 
      g_list_foreach(chain->messages, (GFunc) g_free, NULL);
      g_list_free(chain->messages);
      chain->messages = NULL;
 }
 
-void error_flush(int context)
+void
+error_flush(int context)
 {
      GtkWidget *pixmap, *popupwin, *vbox, *vbox1, *hbox0, *hbox, *vbox2, *msg_label, *okbutton, *align;
      struct errchain *chain;
      GList *msg;
 
+	if(!context) {
+		return;
+	}
+
      chain = error_chain_by_context(context);
      g_assert(chain);
 
@@ -226,7 +293,7 @@ void error_flush(int context)
 					    GTK_WINDOW(chain->transient_for));
 	  }
 
-	  gtk_widget_realize(popupwin);
+	  //gtk_widget_realize(popupwin);
 	  gtk_window_set_title(GTK_WINDOW(popupwin), chain->title);
 	  gtk_window_set_policy(GTK_WINDOW(popupwin), FALSE, FALSE, FALSE);
 	  vbox1 = GTK_DIALOG(popupwin)->vbox;
@@ -265,6 +332,7 @@ void error_flush(int context)
 	       gtk_box_pack_start(GTK_BOX(vbox), msg_label, FALSE, FALSE, 0);
 
 	       g_free(msg->data);
+	       msg->data = NULL;
 	  }
 	  g_list_free(chain->messages);
 	  chain->messages = NULL;
@@ -277,7 +345,7 @@ void error_flush(int context)
 	  gtk_box_pack_end(GTK_BOX(vbox2), hbox0, TRUE, FALSE, 0);
 	  gtk_widget_show(hbox0);
 
-	  okbutton = gtk_button_new_from_stock(GTK_STOCK_OK);
+	  okbutton = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
 	  g_signal_connect_swapped(okbutton, "clicked",
 				    G_CALLBACK(gtk_widget_destroy),
 				    GTK_OBJECT(popupwin));
@@ -288,27 +356,22 @@ void error_flush(int context)
 	  gtk_box_pack_end(GTK_BOX(hbox0), okbutton, TRUE, FALSE, 0);
 	  gtk_widget_grab_default(okbutton);
 	  gtk_widget_show(okbutton);
-	  gtk_widget_show(popupwin);
+	  if(!gq_main_loop_blocked()) {
+	  	gtk_dialog_run(GTK_DIALOG(popupwin));
+	  } else {
+		  gtk_widget_show(popupwin);
+	  }
      }
 
      error_free(context);
 }
 
 
-void error_popup(char *title, char *message, GtkWidget *transient_for)
+void
+error_popup(char *title, char *message, GtkWidget *transient_for)
 {
-     int context;
-
-     context = error_new_context(title, transient_for);
+     int context = error_new_context(title, transient_for);
      error_push(context, message);
      error_flush(context);
-
 }
 
-
-
-/* 
-   Local Variables:
-   c-basic-offset: 5
-   End:
- */
