@@ -31,67 +31,97 @@
 #include <errno.h>		/* errno */
 #include <stdio.h>		/* FILE */
 #include <stdlib.h>		/* free - MUST get rid of malloc/free */
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 # include  <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include "browse-dnd.h"		/* copy_entry et al */
 #include "common.h"
+#include "configfile.h"		/* config */
+#include "encode.h"
+#include "errorchain.h"
+#include "gq-browser-model.h"
 #include "gq-browser-node-reference.h"
+#include "gq-export-dialog.h"
+#include "gq-formfill.h"	/* formlist_from_entry */
+#include "gq-input-form.h"	/* new_from_entry */
 #include "gq-tab-browse.h"
 #include "gq-tab-search.h"
-
-#include "input.h"		/* new_from_entry */
+#include "gq-utilities.h"
+#include "gq-window.h"
 #include "search.h"		/* fill_out_search */
 #include "template.h"		/* struct gq_template */
-#include "formfill.h"		/* formlist_from_entry */
-
 #include "tinput.h"		/* formfill_from_template */
-#include "browse-dnd.h"		/* copy_entry et al */
 
-#include "configfile.h"		/* config */
-#include "errorchain.h"
-#include "util.h"
-#include "encode.h"
+struct GqBrowserNodeDnPrivate {
+	GqServerDn* entry;
+};
+#define P(i) (G_TYPE_INSTANCE_GET_PRIVATE((i), GQ_TYPE_BROWSER_NODE_DN, struct GqBrowserNodeDnPrivate))
 
-#include "browse-export.h"
+gchar const*
+gq_browser_node_dn_get_dn(GqBrowserNodeDn const* self)
+{
+	g_return_val_if_fail(GQ_IS_BROWSER_NODE_DN(self), NULL);
 
-static void tree_row_search_below(GtkMenuItem *menuitem, GqTab *tab)
+	return gq_server_dn_get_dn(P(self)->entry);
+}
+
+GqServerDn*
+gq_browser_node_dn_get_entry(GqBrowserNodeDn const* self)
+{
+	g_return_val_if_fail(GQ_IS_BROWSER_NODE_DN(self), NULL);
+
+	return P(self)->entry;
+}
+
+static void
+tree_row_search_below(GqTab* tab)
 {
      GQTreeWidget *ctree;
-     GQTreeWidgetNode *node;
      GqBrowserNode *e;
      GqServer *server;
      GqTab *search_tab;
 
      ctree = GQ_TAB_BROWSE(tab)->ctreeroot;
-     node = GQ_TAB_BROWSE(tab)->tree_row_popped_up;
-     e = GQ_BROWSER_NODE(gq_tree_get_node_data (ctree, node));
+     e = gq_browser_model_get_node(gtk_tree_view_get_model(ctree),
+		                   &GQ_TAB_BROWSE(tab)->tree_row_popped_up);
 
      g_assert(GQ_IS_BROWSER_NODE_DN(e));
 
-     server = server_from_node(ctree, node);
+     server = gq_browser_node_get_server(e);
 
      if (e == NULL || server == NULL)
 	  return;
 
-     search_tab = get_last_of_mode(SEARCH_MODE);
-     if (!search_tab) {
-	  new_modetab(&mainwin, SEARCH_MODE);
-	  search_tab = get_last_of_mode(SEARCH_MODE);
-     }
+	search_tab = gq_window_get_last_tab(&mainwin, GQ_TYPE_TAB_SEARCH);
+	if (!search_tab) {
+#warning "FIXME: this needs an API cleanup"
+		new_modetab(&mainwin, GQ_TYPE_TAB_SEARCH, TRUE);
+		search_tab = gq_window_get_last_tab(&mainwin, GQ_TYPE_TAB_SEARCH);
+	}
 
-     if (search_tab) {
-	  fill_out_search(search_tab, server, GQ_BROWSER_NODE_DN(e)->dn);
-     }
+	g_return_if_fail(search_tab);
+	fill_out_search(search_tab, server,
+			gq_browser_node_dn_get_dn(GQ_BROWSER_NODE_DN(e)));
 }
 
-static GQTreeWidgetNode *ref_browse_single_add(const char *uri,
-					   GQTreeWidget *ctree,
-					   GQTreeWidgetNode *node)
+#ifndef USE_TREE_VIEW
+static GQTreeWidgetNode*
+ref_browse_single_add(gchar const     *uri,
+		      GQTreeWidget    * ctree,
+		      GQTreeWidgetNode* node)
+#else
+static void
+ref_browse_single_add(gchar const* uri,
+		      GtkTreeModel* model,
+		      GtkTreeIter * parent)
+#endif
 {
+#ifndef USE_TREE_VIEW
      GqBrowserNodeReference *new_entry;
-     GQTreeWidgetNode *new_item, *added = NULL;
+     GQTreeWidgetNode *added = NULL;
 
      new_entry = GQ_BROWSER_NODE_REFERENCE(gq_browser_node_reference_new(uri));
 
@@ -105,26 +135,35 @@ static GQTreeWidgetNode *ref_browse_single_add(const char *uri,
 				added);
 
      return added;
+#else
+	g_message("FIXME: implement with GtkTreeView");
+	return;
+#endif
 }
 
 
-static void browse_new_from_entry_callback(GtkMenuItem *widget,
-					   GqBrowserNodeDn *entry)
+static void
+browse_new_from_entry_callback(GtkMenuItem    * widget,
+			       GqBrowserNodeDn* entry)
 {
-     if (GQ_IS_BROWSER_NODE_DN(entry)) {
-	  char *dn = entry->dn;
-	  int error_context =
+	GqServer   * server;
+	gchar const* dn;
+	int error_context;
+
+	g_return_if_fail(GQ_IS_BROWSER_NODE_DN(entry));
+
+	dn = gq_browser_node_dn_get_dn(entry);
+	error_context =
 	       error_new_context(_("Creating new entry from existing entry"),
 				 GTK_WIDGET(widget));
-
-	  GqServer *server =
+#warning "FIXME: use the GqServerDn, not this beast"
+	server =
 	       GQ_SERVER(gtk_object_get_data(GTK_OBJECT(widget),
 							 "server"));
 
-	  new_from_entry(error_context, server, dn);
+	  new_from_entry(server, dn);
 
 	  error_flush(error_context);
-     }
 }
 
 static void browse_new_from_template_callback(GtkWidget *widget,
@@ -133,7 +172,7 @@ static void browse_new_from_template_callback(GtkWidget *widget,
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
      GList *formlist;
      GqServer *server;
-     struct inputform *iform;
+     GqInputForm *iform;
      GqBrowserNodeDn *entry;
      int error_context;
 
@@ -147,51 +186,47 @@ static void browse_new_from_template_callback(GtkWidget *widget,
 	  error_new_context(_("Creating now entry from template"),
 			    widget);
 
-     iform = new_inputform();
-     iform->dn = NULL;
-     iform->server = g_object_ref(server);
+     iform = GQ_INPUT_FORM(gq_input_form_new());
 
      formlist = formfill_from_template(error_context, server, template);
      if(formlist) {
-	  iform->formlist = formlist;
-	  if (entry && entry->dn) {
-	       /* don't need the RDN of the current entry */
-	       char *newdn = g_malloc(strlen(entry->dn) + 2);
-	       newdn[0] = ',';
-	       newdn[1] = 0;
-	       strcat(newdn, entry->dn);
-	       iform->dn = newdn;
+	  gq_input_form_set_formlist(iform, formlist);
+	  if (entry && gq_browser_node_dn_get_dn(entry)) {
+#warning "FIXME: provide a nicer default RDN"
+			/* don't need the RDN of the current entry */
+			char *newdn = g_strdup_printf(",%s", gq_browser_node_dn_get_dn(entry));
+			GqServerDn* _entry = gq_server_dn_new(newdn, server);
+			gq_input_form_set_entry(iform, _entry);
+			g_object_unref(_entry);
+			g_free(newdn);
 	  }
 
 	  create_form_window(iform);
-	  create_form_content(iform);
-
-	  build_inputform(error_context, iform);
-     } else {
-	  free_inputform(iform);
      }
 
+	g_object_unref(iform);
      error_flush(error_context);
 #endif /* HAVE_LDAP_STR2OBJECTCLASS */
 }
 
-static void dump_subtree(GtkWidget *widget, GqTab *tab)
+static void
+dump_subtree(GtkWidget* widget G_GNUC_UNUSED,
+	     GqTab    * tab)
 {
      GQTreeWidget *ctree;
-     GQTreeWidgetNode *node;
      GqBrowserNode *e;
      GqServer *server;
      GList *to_export = NULL;
-     struct dn_on_server *dos;
+     GqServerDn *dos;
      int error_context;
 
      ctree = GQ_TAB_BROWSE(tab)->ctreeroot;
-     node = GQ_TAB_BROWSE(tab)->tree_row_popped_up;
-     e = GQ_BROWSER_NODE(gq_tree_get_node_data (ctree, node));
+     e = gq_browser_model_get_node(gtk_tree_view_get_model(ctree),
+		     		   &GQ_TAB_BROWSE(tab)->tree_row_popped_up);
 
      g_assert(GQ_IS_BROWSER_NODE_DN(e));
 
-     server = server_from_node(ctree, node);
+     server = gq_browser_node_get_server(e);
 
      if (e == NULL || server == NULL)
 	  return;
@@ -199,16 +234,18 @@ static void dump_subtree(GtkWidget *widget, GqTab *tab)
      error_context = error_new_context(_("Exporting entry to LDIF"),
 				       tab->win->mainwin);
 
-     dos = new_dn_on_server(GQ_BROWSER_NODE_DN(e)->dn, server);
+     dos = gq_server_dn_new(gq_browser_node_dn_get_dn(GQ_BROWSER_NODE_DN(e)), server);
      dos->flags = LDAP_SCOPE_SUBTREE; /* default is LDAP_SCOPE_BASE */
      to_export = g_list_append(to_export, dos);
 
-     export_many(error_context, tab->win->mainwin, to_export);
+     gq_export_server_dns(error_context, GTK_WINDOW(tab->win->mainwin), to_export);
 
      error_flush(error_context);
 }
 
-static void delete_browse_entry(GtkWidget *widget, GqTab *tab)
+static void
+delete_browse_entry(GtkWidget* widget G_GNUC_UNUSED,
+		    GqTab    * tab)
 {
      GQTreeWidget *ctree;
      GQTreeWidgetNode *node;
@@ -217,22 +254,24 @@ static void delete_browse_entry(GtkWidget *widget, GqTab *tab)
      int do_delete;
 
      ctree = GQ_TAB_BROWSE(tab)->ctreeroot;
-     node = GQ_TAB_BROWSE(tab)->selected_ctree_node;
 
-     entry = GQ_BROWSER_NODE_DN(gq_tree_get_node_data (ctree, node));
-     if (entry == NULL)
-	  return;
+	entry = gq_browser_model_get_node(gtk_tree_view_get_model(ctree),
+					  &GQ_TAB_BROWSE(tab)->tree_row_popped_up);
+	if (entry == NULL)
+		return;
 
-     if ((server = server_from_node(ctree, node)) == NULL)
-	  return;
+	server = gq_browser_node_get_server(entry);
+	if (server == NULL)
+		return;
 
      do_delete = 0;
-
+#ifndef USE_TREE_VIEW
      gtk_clist_freeze(GTK_CLIST(ctree));
      
      if (!entry->seen) {
 	  gq_tree_fire_expand_callback (ctree, node);
      }
+#endif
 
      if (entry->leaf) {
 	  /* item is a leaf node */
@@ -248,7 +287,9 @@ static void delete_browse_entry(GtkWidget *widget, GqTab *tab)
 	  */
 
 	  do_delete =
-	       question_popup(_("Warning"),
+#warning "FIXME: set a parent window"
+	       question_popup(NULL,
+			      _("Warning"),
 			      _("This entry has a subtree!\n"
 				"Do you want to delete every entry under it as well?"));
      }
@@ -257,7 +298,7 @@ static void delete_browse_entry(GtkWidget *widget, GqTab *tab)
      if (do_delete) {
 	  int ctx = error_new_context(_("Deleting entry/subtree"),
 				      GTK_WIDGET(ctree));
-	  if (delete_entry_full(ctx, server, entry->dn, TRUE)) {
+	  if (delete_entry_full(ctx, server, gq_browser_node_dn_get_dn(entry))) {
 	       GqBrowserNode *p_entry;
 	       GQTreeWidgetNode *parent = gq_tree_get_parent_node (ctree,
 								   node);
@@ -280,36 +321,19 @@ static void delete_browse_entry(GtkWidget *widget, GqTab *tab)
      gtk_clist_thaw(GTK_CLIST(ctree));
 }
 
-
-
-
-
-
-
-
-
-/*
- * Destructor for GqBrowserNodeDn objects
- */
-static void destroy_dn_browse_entry(GqBrowserNode *e)
-{
-     GqBrowserNodeDn *entry;
-
-     if (!e) return;
-
-     g_assert(GQ_IS_BROWSER_NODE_DN(e));
-     entry = GQ_BROWSER_NODE_DN(e);
-
-     if (entry->dn) g_free(entry->dn);
-     free(entry);
-}
-
-
-static void dn_browse_entry_expand(GqBrowserNode *be,
-				   int error_context,
-				   GQTreeWidget *ctree,
-				   GQTreeWidgetNode *node,
-				   GqTab *tab)
+static void
+dn_browse_entry_expand(GqBrowserNode   * be,
+		       int               error_context,
+#ifndef USE_TREE_VIEW
+		       GQTreeWidget    * ctree,
+		       GQTreeWidgetNode* node,
+		       GqTab           * tab G_GNUC_UNUSED
+#else
+		       GtkTreeModel * model,
+		       GtkTreeIter  * iter,
+		       gpointer       _dummy G_GNUC_UNUSED
+#endif
+		       )
 {
      LDAP *ld = NULL;
      LDAPMessage *res = NULL, *e;
@@ -328,17 +352,21 @@ static void dn_browse_entry_expand(GqBrowserNode *be,
      entry = GQ_BROWSER_NODE_DN(be);
 
      if (!entry->seen) {
-	  server = server_from_node(ctree, node);
+	  server = server_from_node(model, iter);
 /*  	  printf("server=%08lx host=%s dn=%s\n", (long) server, */
 /*  		 server->ldaphost, */
 /*  		 entry->dn); */
-	  
+#ifndef USE_TREE_VIEW
 	  gtk_clist_freeze(GTK_CLIST(ctree));
-
 	  gq_tree_remove_children (ctree, node);
+#else
+		g_message("FIXME: implement with GtkTreeView");
+#endif
 
 	  if( (ld = open_connection(error_context, server)) == NULL) {
+#ifndef USE_TREE_VIEW
 	       gtk_clist_thaw(GTK_CLIST(ctree));
+#endif
 	       return;
 	  }
 
@@ -349,7 +377,7 @@ static void dn_browse_entry_expand(GqBrowserNode *be,
 	  }
 #endif
 
-	  statusbar_msg(_("Onelevel search on %s"), entry->dn);
+	  statusbar_msg(_("Onelevel search on %s"), gq_browser_node_dn_get_dn(entry));
 	  
 	  ct.ldctl_oid		= LDAP_CONTROL_MANAGEDSAIT;
 	  ct.ldctl_value.bv_val	= NULL;
@@ -360,7 +388,7 @@ static void dn_browse_entry_expand(GqBrowserNode *be,
 	  
 	  /* check if this is a referral object */
 
-	  rc = ldap_search_ext(ld, entry->dn,
+	  rc = ldap_search_ext(ld, gq_browser_node_dn_get_dn(entry),
 			       LDAP_SCOPE_BASE, 
 			       "(objectClass=referral)", ref, 0,
 			       ctrls,		/* serverctrls */
@@ -393,7 +421,11 @@ static void dn_browse_entry_expand(GqBrowserNode *be,
 		    
 		    for(i = 0; vals[i]; i++) {
 			 entry->is_ref = TRUE; /* now we know for sure */
+#ifndef USE_TREE_VIEW
 			 ref_browse_single_add(vals[i], ctree, node);
+#else
+			ref_browse_single_add(vals[i], model, iter);
+#endif
 		    }
 		    
 		    if (vals)  /* redundant, but... */
@@ -409,7 +441,9 @@ static void dn_browse_entry_expand(GqBrowserNode *be,
 	  if (entry->is_ref) {
 	       entry->seen = TRUE;
 	       statusbar_msg(_("Showing referrals"));
+#ifndef USE_TREE_VIEW
 	       gtk_clist_thaw(GTK_CLIST(ctree));
+#endif
 	       close_connection(server, FALSE);
 	       return;
 	  }
@@ -418,7 +452,7 @@ static void dn_browse_entry_expand(GqBrowserNode *be,
 
 
 
-	  rc = ldap_search_ext(ld, entry->dn,
+	  rc = ldap_search_ext(ld, gq_browser_node_dn_get_dn(entry),
 			       LDAP_SCOPE_ONELEVEL, 
 			       "(objectClass=*)", dummy, 1,
 			       ctrls,		/* serverctrls */
@@ -447,7 +481,11 @@ static void dn_browse_entry_expand(GqBrowserNode *be,
 		   e = ldap_next_entry(ld, e)) {
 
 		    char *dn = ldap_get_dn(ld, e);
-		    dn_browse_single_add(dn, ctree, node);
+#ifndef USE_TREE_VIEW
+		    dn_browse_single_add(gq_server_dn_get_server(P(entry)->entry), dn, ctree, node);
+#else
+		    dn_browse_single_add(gq_server_dn_get_server(P(entry)->entry), dn, model, iter, NULL);
+#endif
 		    if (dn) ldap_memfree(dn);
 
 		    num_children++;
@@ -462,11 +500,16 @@ static void dn_browse_entry_expand(GqBrowserNode *be,
 	       }
 	       ldap_msgfree(res);
 	  }
+
+#ifndef USE_TREE_VIEW
 	  /* tree sorting */
 	  gtk_clist_set_sort_type(GTK_CLIST(ctree), GTK_SORT_ASCENDING);
 	  gtk_clist_set_sort_column(GTK_CLIST(ctree), 0);
 	  gtk_clist_set_compare_func(GTK_CLIST(ctree), (GtkCListCompareFunc)NULL);
 	  gq_tree_widget_sort_node(GQ_TREE_WIDGET(ctree), node);
+#else
+		g_message("FIXME: implement with GtkTreeView");
+#endif
 	  entry->leaf = (num_children == 0);
 
 	  g_snprintf(message, sizeof(message),
@@ -478,7 +521,9 @@ static void dn_browse_entry_expand(GqBrowserNode *be,
 
 	  if (rc == LDAP_SERVER_DOWN) {
 	       server->server_down++;
+#ifndef USE_TREE_VIEW
 	       gtk_clist_thaw(GTK_CLIST(ctree));
+#endif
 	       goto done;
 	  }
 
@@ -522,8 +567,9 @@ static void dn_browse_entry_expand(GqBrowserNode *be,
 
 	  statusbar_msg(message);
 
+#ifndef USE_TREE_VIEW
 	  gtk_clist_thaw(GTK_CLIST(ctree));
-
+#endif
 	  entry->seen = TRUE;
      }
 
@@ -545,21 +591,44 @@ static void dn_browse_entry_expand(GqBrowserNode *be,
      if (server && ld) close_connection(server, FALSE);
 }
 
-static void browse_edit_from_entry(GqBrowserNode *e,
-				   int error_context,
-				   GQTreeWidget *ctreeroot,
-				   GQTreeWidgetNode *ctreenode,
-				   GqTab *tab)
+static void
+browse_edit_from_entry(GqBrowserNode *e,
+		       int error_context,
+#ifndef USE_TREE_VIEW
+		       GQTreeWidget *ctreeroot,
+		       GQTreeWidgetNode *ctreenode,
+#else
+		       GtkTreeModel* model,
+		       GtkTreeIter * iter,
+#endif
+		       GqTab *tab)
 {
-     GList *oldlist, *newlist, *tmplist;
-     GtkWidget *pane2_scrwin, *pane2_vbox;
+#ifdef USE_TREE_VIEW
+	GtkWidget* input_form;
+
+	g_return_if_fail(GQ_IS_BROWSER_MODEL(model));
+	g_return_if_fail(gq_browser_model_validate(GQ_BROWSER_MODEL(model), iter));
+
+	input_form = gq_input_form_new();
+	gtk_widget_show(input_form);
+	gq_tab_browse_set_input_form(GQ_TAB_BROWSE(tab), input_form);
+	gq_input_form_set_editable(GQ_INPUT_FORM(input_form), TRUE);
+
+	/* pass on old hide status */
+#warning "FIXME: the browse tab should manage this"
+	gq_input_form_set_hide(GQ_INPUT_FORM(input_form),
+			       GQ_TAB_BROWSE(tab)->hidden);
+
+	gq_input_form_set_entry(GQ_INPUT_FORM(input_form), P(e)->entry);
+#else
+	GList *oldlist;
      GqServer *server;
-     struct inputform *iform;
-     char *dn;
+     GqInputForm *iform;
+	gchar const*dn;
 /*      int hidden = 0; */
      GqBrowserNodeDn *entry;
 
-     g_assert(GQ_IS_BROWSER_NODE_DN(e));
+	g_return_if_fail(GQ_IS_BROWSER_NODE_DN(e));
      entry = GQ_BROWSER_NODE_DN(e);
 
      if (ctreenode == NULL)
@@ -568,60 +637,40 @@ static void browse_edit_from_entry(GqBrowserNode *e,
      if( (server = server_from_node(ctreeroot, ctreenode)) == NULL)
 	  return;
 
-     dn = entry->dn;
-     record_path(tab, GQ_BROWSER_NODE(entry), ctreeroot, ctreenode);
+	dn = gq_browser_node_dn_get_dn(GQ_BROWSER_NODE_DN(entry));
+     record_path(tab, ctreeroot, ctreenode);
 
      ctreeroot = GQ_TAB_BROWSE(tab)->ctreeroot;
 
-     iform = new_inputform();
-     GQ_TAB_BROWSE(tab)->inputform = iform;
+	iform = GQ_INPUT_FORM(gq_input_form_new());
+	gtk_widget_show(GTK_WIDGET(iform));
+	gq_tab_browse_set_input_form(GQ_TAB_BROWSE(tab), iform);
+	gq_input_form_set_editable(iform, TRUE);
 
-     iform->server = g_object_ref(server);
+	/* pass on old "hide" status */
+#warning "FIXME: there should be a nicer way"
+	gq_input_form_set_hide(iform, GQ_TAB_BROWSE(tab)->hidden);
 
-     iform->edit = 1;
-
-     /* pass on old "hide" status */
-     iform->hide_status = GQ_TAB_BROWSE(tab)->hidden;
-
-     tmplist = NULL;
      oldlist = formlist_from_entry(error_context, server, dn, 0);
-#ifdef HAVE_LDAP_STR2OBJECTCLASS
-     oldlist = add_schema_attrs(error_context, server, oldlist);
-#endif
 
      if(oldlist) {
-	  iform->oldlist = oldlist;
-	  newlist = dup_formlist(oldlist);
-	  iform->formlist = newlist;
-	  iform->olddn = g_strdup(dn);
-	  iform->dn = g_strdup(dn);
+#warning "FIXME: check whether the old* things are necessary"
+		GqServerDn* _entry = gq_server_dn_new(dn, server);
+	  gq_input_form_set_oldlist(iform, oldlist);
+	  gq_input_form_set_formlist(iform, dup_formlist(oldlist));
+	  gq_input_form_set_old_dn(iform, dn);
+		gq_input_form_set_entry(iform, _entry);
 
 	  if (ctreeroot) {
-	       iform->ctreeroot = ctreeroot;
-	       iform->ctree_refresh = gq_tree_get_parent_node (ctreeroot,
-							       ctreenode);
+	       gq_input_form_set_ctree_root(iform, ctreeroot);
+	       gq_input_form_set_ctree_refresh(iform, gq_tree_get_parent_node(ctreeroot, ctreenode));
 	  }
 
-	  /* XXX should free etc first */
-	  pane2_scrwin = GQ_TAB_BROWSE(tab)->pane2_scrwin;
-	  gtk_container_remove(GTK_CONTAINER(pane2_scrwin),
-			       GTK_BIN(pane2_scrwin)->child);
-
-	  pane2_vbox = gtk_vbox_new(FALSE, 2);
-	  iform->target_vbox = pane2_vbox;
-	  GQ_TAB_BROWSE(tab)->pane2_vbox = pane2_vbox;
-
-	  gtk_widget_show(pane2_vbox);
-	  gtk_widget_set_parent_window(pane2_vbox, (mainwin.mainwin->window));
-	  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pane2_scrwin), pane2_vbox);
-
-	  create_form_content(iform);
-	  build_inputform(error_context, iform);
-     } else {
-	  inputform_free(iform);
-	  GQ_TAB_BROWSE(tab)->inputform = NULL;
-     }
-
+		g_object_unref(_entry);
+	} else {
+		gq_tab_browse_set_input_form(GQ_TAB_BROWSE(tab), NULL);
+	}
+#endif
 }
 
 static void dn_browse_entry_refresh(GqBrowserNode *entry,
@@ -638,18 +687,22 @@ static void dn_browse_entry_refresh(GqBrowserNode *entry,
 						  ctree, node, tab);
 }
 
-static char* dn_browse_entry_get_name(GqBrowserNode *entry,
+static char* dn_browse_entry_get_name(GqBrowserNode const*entry,
 				      gboolean long_form)
 {
-     char **exploded_dn;
-     char *g;
+	gchar const* dn;
 
-     g_assert(GQ_IS_BROWSER_NODE_DN(entry));
+	g_assert(GQ_IS_BROWSER_NODE_DN(entry));
+
+	dn = gq_browser_node_dn_get_dn(GQ_BROWSER_NODE_DN(entry));
 
      if (long_form) {
-	  return g_strdup(GQ_BROWSER_NODE_DN(entry)->dn);
+	  return g_strdup(dn);
      } else {
-	  exploded_dn = gq_ldap_explode_dn(GQ_BROWSER_NODE_DN(entry)->dn, FALSE);
+	  char **exploded_dn;
+	  char *g;
+
+	  exploded_dn = gq_ldap_explode_dn(dn, FALSE);
 
 	  g = g_strdup(exploded_dn[0]);
 
@@ -672,12 +725,14 @@ static void dn_browse_entry_popup(GqBrowserNode *entry, GtkWidget *menu,
      struct gq_template *template;
 #endif
 
-     g_assert(GQ_IS_BROWSER_NODE_DN(entry));
+	g_return_if_fail(GQ_IS_BROWSER_NODE_DN(entry));
 
      is_dn = GQ_IS_BROWSER_NODE_DN(entry);
-     
-     if ((server = server_from_node(ctreeroot, ctree_node)) == NULL)
-	  return;
+
+	server = gq_browser_node_get_server(entry);
+	if(server == NULL) {
+		return;
+	}
      
      /* New submenu */
      menu_item = gtk_menu_item_new_with_label(_("New"));
@@ -713,8 +768,10 @@ static void dn_browse_entry_popup(GqBrowserNode *entry, GtkWidget *menu,
 			entry);
      gtk_widget_show(menu_item);
 
-     /* Export to LDIF */
-     menu_item = gtk_menu_item_new_with_label(_("Export to LDIF"));
+	/* Export to LDIF */
+	menu_item = gtk_image_menu_item_new_with_label(_("Export to LDIF"));
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),
+				      gtk_image_new_from_stock(GTK_STOCK_SAVE_AS, GTK_ICON_SIZE_MENU));
      gtk_menu_append(GTK_MENU(menu), menu_item);
      g_signal_connect(menu_item, "activate",
 			G_CALLBACK(dump_subtree),
@@ -728,9 +785,8 @@ static void dn_browse_entry_popup(GqBrowserNode *entry, GtkWidget *menu,
      /* Search below */
      menu_item = gtk_menu_item_new_with_label(_("Search below"));
      gtk_menu_append(GTK_MENU(menu), menu_item);
-     g_signal_connect(menu_item, "activate",
-			G_CALLBACK(tree_row_search_below),
-			tab);
+	g_signal_connect_swapped(menu_item, "activate",
+				 G_CALLBACK(tree_row_search_below), tab);
      gtk_widget_show(menu_item);
 
      menu_item = gtk_menu_item_new();
@@ -799,29 +855,101 @@ static void dn_browse_entry_popup(GqBrowserNode *entry, GtkWidget *menu,
  * Constructor for GqBrowserNodeDn objects taking the dn
  */
 GqBrowserNode*
-gq_browser_node_dn_new(const char *dn)
+gq_browser_node_dn_new(GqServerDn* entry)
 {
-	GqBrowserNodeDn *e = g_object_new(GQ_TYPE_BROWSER_NODE_DN, NULL);
+#warning "FIXME: take the entry and pass it to g_object_new()"
+	GqBrowserNode *self = g_object_new(GQ_TYPE_BROWSER_NODE_DN, NULL);
+	gchar * const *classes;
+	static gchar const* types[] = {
+		"folder",
+		"ldap-organization",
+		"ldap-group",
+		"ldap-person"
+	};
+	guint type = 0; // index into types[]
 
-	e->dn = g_strdup(dn);
+	P(self)->entry = g_object_ref(entry);
+	classes = gq_server_dn_get_object_class(P(self)->entry);
+	for(; classes && *classes; classes++) {
+		if(!strcmp("organization", *classes)) {
+			type = MAX(type, 1);
+		}
+		else if(!strcmp(*classes, "groupOfNames") ||
+			!strcmp(*classes, "posixGroup"))
+		{
+			type = MAX(type, 2);
+		}
+		else if(!strcmp(*classes, "account") ||
+			!strcmp(*classes, "mailUser") ||
+			!strcmp(*classes, "organizationalPerson") ||
+			!strcmp(*classes, "person") ||
+			!strcmp(*classes, "pilotPerson") ||
+			!strcmp(*classes, "newPilotPerson") ||
+			!strcmp(*classes, "pkiUser") ||
+			!strcmp(*classes, "posixUser") ||
+			!strcmp(*classes, "posixAccount") ||
+			!strcmp(*classes, "residentalPerson") ||
+			!strcmp(*classes, "shadowAccount") ||
+			!strcmp(*classes, "strongAuthenticationUser") ||
+			!strcmp(*classes, "inetOrgPerson"))
+		{
+			type = MAX(type, 3);
+		}
+	}
 
-	return GQ_BROWSER_NODE(e);
+#ifndef G_DISABLE_CHECKS
+	if(type >= G_N_ELEMENTS(types)) {
+		g_warning("Invalid type index");
+		return self;
+	}
+#endif
+	gq_browser_node_set_status(GQ_BROWSER_NODE(self), types[type]);
+
+	return self;
 }
 
 /* GType */
 G_DEFINE_TYPE(GqBrowserNodeDn, gq_browser_node_dn, GQ_TYPE_BROWSER_NODE);
 
 static void
-gq_browser_node_dn_init(GqBrowserNodeDn* self) {}
+gq_browser_node_dn_init(GqBrowserNodeDn* self G_GNUC_UNUSED)
+{}
 
 static void
-gq_browser_node_dn_class_init(GqBrowserNodeDnClass* self_class) {
-	GqBrowserNodeClass* node_class = GQ_BROWSER_NODE_CLASS(self_class);
-	node_class->destroy  = destroy_dn_browse_entry;
+browser_node_dn_dispose(GObject* object)
+{
+	GqBrowserNodeDn* self = GQ_BROWSER_NODE_DN(object);
+
+	if(P(self)->entry) {
+		g_object_unref(P(self)->entry);
+		P(self)->entry = NULL;
+	}
+
+	G_OBJECT_CLASS(gq_browser_node_dn_parent_class)->dispose(object);
+}
+
+static GqServer*
+browser_node_dn_get_server(GqBrowserNode* node)
+{
+	return gq_server_dn_get_server(P(node)->entry);
+}
+
+static void
+gq_browser_node_dn_class_init(GqBrowserNodeDnClass* self_class)
+{
+	GObjectClass      * object_class = G_OBJECT_CLASS(self_class);
+	GqBrowserNodeClass* node_class   = GQ_BROWSER_NODE_CLASS(self_class);
+
+	object_class->dispose = browser_node_dn_dispose;
+
 	node_class->expand   = dn_browse_entry_expand;
 	node_class->select   = browse_edit_from_entry;
 	node_class->refresh  = dn_browse_entry_refresh;
 	node_class->get_name = dn_browse_entry_get_name;
 	node_class->popup    = dn_browse_entry_popup;
+	node_class->get_server = browser_node_dn_get_server;
+
+	/* GqBrowserNodeDnClass */
+	g_type_class_add_private(self_class, sizeof(struct GqBrowserNodeDnPrivate));
 }
 
