@@ -24,16 +24,16 @@
 #ifndef GQ_KEYRING_H
 #define GQ_KEYRING_H
 
-#include <gnome-keyring.h>
-#include "common.h" // for ldapserver
+#include "gq-server.h"
 
 G_BEGIN_DECLS
 
-GnomeKeyringAttributeList*
-	gq_keyring_attribute_list_from_server(GqServer* server);
-void    gq_keyring_forget_password           (GqServer* server);
-gchar*	gq_keyring_get_password		     (GqServer* server);
-void	gq_keyring_save_server_password      (GqServer* server);
+gboolean gq_keyring_can_save       (void);
+void     gq_keyring_forget_password(GqServer const* server);
+void     gq_keyring_free_password  (gchar         * password);
+gchar*	 gq_keyring_get_password   (GqServer const* server);
+void	 gq_keyring_save_password  (GqServer const* server,
+				    gchar const   * password);
 
 G_END_DECLS
 
