@@ -39,13 +39,13 @@
 #  endif /* HAVE_MCHECK */
 #endif /* DEBUG */
 
-#include "gq-server-list.h"
-#include "mainwin.h"
 #include "configfile.h"
-#include "syntax.h"
-#include "encode.h"
 #include "debug.h"
+#include "encode.h"
+#include "gq-server-list.h"
+#include "gq-window.h"
 #include "state.h"
+#include "syntax.h"
 
 static void sigpipehandler(int sig);
 
@@ -163,6 +163,7 @@ main(int argc, char **argv) {
 
      free_config(config);
      config = NULL;
+     g_object_unref(gq_server_list_get());
 
      save_state();
 
@@ -178,11 +179,15 @@ main(int argc, char **argv) {
 }
 
 static void
-increase_down_per_server(GQServerList* list, GqServer* server, gpointer user_data) {
+increase_down_per_server(GQServerList* list G_GNUC_UNUSED,
+			 GqServer    * server,
+			 gpointer      user_data G_GNUC_UNUSED)
+{
 	server->server_down++;
 }
 
-static void sigpipehandler(int sig)
+static void
+sigpipehandler(int sig G_GNUC_UNUSED)
 {
 #ifdef DEBUG
      if (debug & GQ_DEBUG_SIGNALS) {
