@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: state.c 975 2006-09-07 18:44:41Z herzi $ */
+#include "state.h"
 
 #ifdef HAVE_CONFIG_H
 # include  <config.h>
@@ -39,13 +39,12 @@
 #include <glib/gmessages.h>
 #include <gtk/gtk.h>
 
-#include "xmlparse.h"
 #include "configfile.h"
-#include "util.h"
-#include "errorchain.h"
-#include "xmlutil.h"
-#include "state.h"
 #include "encode.h"		/* gq_codeset */
+#include "errorchain.h"
+#include "gq-utilities.h"
+#include "xmlparse.h"
+#include "xmlutil.h"
 
 enum state_value_type {
     SV_int = 1,
@@ -136,7 +135,10 @@ static struct state_entity *new_state_entity()
     return e;
 }
 
-static gboolean ghr_free_value(char *key, struct state_value *v, gpointer ud)
+static gboolean
+ghr_free_value(char *key,
+	       struct state_value *v,
+	       gpointer ud G_GNUC_UNUSED)
 {
     g_free(key);
     free_state_value(v);
@@ -146,7 +148,8 @@ static gboolean ghr_free_value(char *key, struct state_value *v, gpointer ud)
 void free_state_entity(struct state_entity *e);
 
 static gboolean ghr_free_entity(char *key, 
-				struct state_entity *e, gpointer ud)
+				struct state_entity *e,
+				gpointer ud G_GNUC_UNUSED)
 {
     g_free(key);
     free_state_entity(e);
@@ -219,6 +222,40 @@ struct state_entity *lookup_entity(const char *entity_name)
     return lookup_entity_with_root(entities, entity_name, TRUE);
 }
 
+int
+state_value_get_enum(gchar const* state_name,
+		     gchar const* value_name,
+		     gint         default_value,
+		     GType        enum_type)
+{
+	gchar const* value_nick = state_value_get_string(state_name,
+							 value_name,
+							 NULL);
+
+	if(G_UNLIKELY(!value_nick)) {
+		return default_value;
+	}
+	else {
+		GEnumClass* e_class = g_type_class_ref(enum_type);
+		gint retval = g_enum_get_value_by_nick(e_class, value_nick)->value;
+		g_type_class_unref(e_class);
+		return retval;
+	}
+}
+
+void
+state_value_set_enum(gchar const* state_name,
+		     gchar const* value_name,
+		     gint         enum_value,
+		     GType        enum_type)
+{
+	GEnumClass* e_class = g_type_class_ref(enum_type);
+	state_value_set_string(state_name,
+			       value_name,
+			       g_enum_get_value(e_class, enum_value)->value_nick);
+	g_type_class_unref(e_class);
+}
+
 int state_value_get_int(const char *state_name,
 			const char *value_name,
 			int def) 
@@ -241,9 +278,10 @@ int state_value_get_int(const char *state_name,
     return def;
 }
 
-void state_value_set_int(const char *state_name,
-			 const char *value_name,
-			 int n) 
+void
+state_value_set_int(const char *state_name,
+		    const char *value_name,
+		    int         n) 
 {
     struct state_value *val;
     struct state_entity *e = lookup_entity(state_name);
@@ -307,6 +345,28 @@ void state_value_set_string(const char *state_name,
     }
 }
 
+GType
+state_value_get_type(gchar const* state_value,
+		     gchar const* value_name,
+		     GType        default_type)
+{
+	gchar const* type_name = state_value_get_string(state_value, value_name, NULL);
+
+	if(!type_name) {
+		return default_type;
+	}
+
+	return g_type_from_name(type_name);
+}
+
+void
+state_value_set_type(gchar const* state_value,
+		     gchar const* value_name,
+		     GType        type)
+{
+	state_value_set_string(state_value, value_name, g_type_name(type));
+}
+
 const GList *state_value_get_list(const char *state_name,
 				  const char *value_name)
 {
@@ -420,9 +480,9 @@ void rm_value(const char *state_name)
 }
 
 
-static void width_height_state(GtkWidget *w,
-			       GtkAllocation *allocation,
-			       char *name)
+static void width_height_state(GtkWidget    * w G_GNUC_UNUSED,
+			       GtkAllocation* allocation,
+			       char         * name)
 {
     state_value_set_int(name, "width",  allocation->width);
     state_value_set_int(name, "height", allocation->height);
@@ -574,7 +634,9 @@ static void save_single_entity(const char *state_name,
     config_write_end_tag(wc, "entity");
 }
 
-void save_state() {
+void
+save_state(void)
+{
     int save_context = error_new_context(_("Error saving statefile"), NULL);
     struct writeconfig *wc;
     gchar *statefile = filename_state(save_context);
@@ -635,7 +697,7 @@ void save_state() {
     g_free(tmpstatefile);
 }
 
-static void gq_stateS(struct parser_context *ctx,
+static void gq_stateS(struct parser_context *ctx G_GNUC_UNUSED,
 		      struct tagstack_entry *e)
 {
     e->data = g_hash_table_new(g_str_hash, g_str_equal);
@@ -658,7 +720,7 @@ static void gq_stateE(struct parser_context *ctx,
 }
 
 
-static void entityS(struct parser_context *ctx,
+static void entityS(struct parser_context *ctx G_GNUC_UNUSED,
 		    struct tagstack_entry *e)
 {
     struct state_entity *ent = new_state_entity();
@@ -706,7 +768,7 @@ static void entity_entityE(struct parser_context *ctx,
     }
 }
 
-static void state_valuesS(struct parser_context *ctx,
+static void state_valuesS(struct parser_context *ctx G_GNUC_UNUSED,
 			  struct tagstack_entry *e)
 {
 /*     struct state_entity *ent = peek_tag(ctx->stack, 1)->data; */
@@ -843,7 +905,9 @@ static GHashTable *process_statefile_XML(int error_context,
     return comm.result;
 }
 
-void init_state() {
+void
+init_state(void)
+{
     int load_context = error_new_context(_("Error loading statefile"), NULL);
     gchar *statefile = filename_state(load_context);
 
@@ -857,3 +921,4 @@ void init_state() {
 
     error_flush(load_context);
 }
+
