@@ -22,22 +22,37 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: syntax.h 955 2006-08-31 19:15:21Z herzi $ */
+/* $Id$ */
 
 #ifndef GQ_SYNTAX_H_INCLUDED
 #define GQ_SYNTAX_H_INCLUDED
 
-#include "formfill.h"
+#include <glib/gmacros.h>
+
+G_BEGIN_DECLS
+
+typedef struct syntax_handler GqSyntax;
+
+G_END_DECLS
+
+#include "gq-formfill.h"
 #include "gq-type-display.h"
 
+G_BEGIN_DECLS
+
+#define GQ_IS_SYNTAX(i) ((i) != NULL)
+
+G_END_DECLS
+
 /* This is not as generic as one could wish, but we have to somehow map
    syntaxes to our code... */
 
 struct syntax_handler {
+#warning "FIXME: make a GObject"
      const char *syntax_oid;
      const char *desc;
      int displaytype;
-     int (*displayTypeFunc)(const char *attr, const char *syn_oid);
+     int (*displayTypeFunc)(const char *attr);
      int must_binary;
 };
 
@@ -59,10 +74,10 @@ int show_in_search(int error_context, GqServer *server,
 void init_syntaxes(void);
 
 /* Return a list of _display_type_handler objects selectable by the user */
-GList *get_selectable_displaytypes();
+GList *get_selectable_displaytypes(void);
 
 /* Utility functions */
-LDAPMod *bervalLDAPMod(struct formfill *form, int op, GList *values);
+LDAPMod *bervalLDAPMod(GqFormfill *form, int op, GList *values);
 
 GByteArray *identity(const char *val, int len);
 
